/***
 * Julia: France Telecom's implementation of the Fractal API
 * Copyright (C) 2001-2002 France Telecom R&D
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Contact: Eric.Bruneton@rd.francetelecom.com
 *
 * Author: Eric Bruneton
 */

package org.objectweb.fractal.julia.control.binding;

import org.objectweb.fractal.api.control.BindingController;
import org.objectweb.fractal.api.control.IllegalBindingException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.api.NoSuchInterfaceException;

import org.objectweb.fractal.julia.ComponentInterface;

import java.util.Map;
import java.util.HashMap;

/**
 * TODO.
 * <br>
 * <br>
 * <b>Requirements</b>
 * <ul>
 * <li>TODO.</li>
 * </ul>
 */

public abstract class OptimizedContainerBindingMixin
  implements BindingController
{

  // -------------------------------------------------------------------------
  // Private constructor
  // -------------------------------------------------------------------------

  private OptimizedContainerBindingMixin () {
  }

  // -------------------------------------------------------------------------
  // Fields and methods added and overriden by the mixin class
  // -------------------------------------------------------------------------

  /**
   * TODO.
   */

  public Map fcBindings;

  public String[] listFc () {
    Map fcBindings = getFcBindings();
    return (String[])fcBindings.keySet().toArray(new String[fcBindings.size()]);
  }

  /**
   * TODO.
   */

  public Object lookupFc (final String clientItfName)
    throws NoSuchInterfaceException
  {
    Map fcBindings = getFcBindings();
    Object result = fcBindings.get(clientItfName);
    if (result == fcBindings) {
      result = null;
    }
    return result;
  }

  /**
   * TODO.
   */

  public void bindFc (final String clientItfName, final Object serverItf) throws
    NoSuchInterfaceException,
    IllegalBindingException,
    IllegalLifeCycleException
  {
    Object o = serverItf;
    if (o instanceof ComponentInterface) {
      o = ((ComponentInterface)o).getFcItfImpl();
    }
    if (o != null) {
      _super_bindFc(clientItfName, o);
    }
    Map fcBindings = getFcBindings();
    fcBindings.put(clientItfName, serverItf);
  }

  /**
   * TODO.
   */

  public void unbindFc (final String clientItfName) throws
    NoSuchInterfaceException,
    IllegalBindingException,
    IllegalLifeCycleException
  {
    _super_unbindFc(clientItfName);
    if (fcBindings != null) {
      fcBindings.put(clientItfName, fcBindings);
    }
  }

  /**
   * TODO.
   */

  private Map getFcBindings () {
    if (fcBindings == null) {
      fcBindings = new HashMap();
      String[] names = _super_listFc();
      for (int i = 0; i < names.length; ++i) {
        fcBindings.put(names[i], fcBindings);
      }
    }
    return fcBindings;
  }

  // -------------------------------------------------------------------------
  // Fields and methods required by the mixin class in the base class
  // -------------------------------------------------------------------------

  /**
   * The {@link BindingController#listFc listFc} method overriden by this
   * mixin.
   */

  public abstract String[] _super_listFc ();

  /**
   * The {@link BindingController#lookupFc lookupFc} method overriden by this
   * mixin.
   */

  public abstract Object _super_lookupFc (String clientItfName) throws
    NoSuchInterfaceException;

  /**
   * The {@link BindingController#bindFc bindFc} method overriden by this mixin.
   */

  public abstract void _super_bindFc (String clientItfName, Object serverItf) throws
    NoSuchInterfaceException,
    IllegalBindingException,
    IllegalLifeCycleException;

  /**
   * The {@link BindingController#unbindFc unbindFc} method overriden by this
   * mixin.
   */

  public abstract void _super_unbindFc (String clientItfName) throws
    NoSuchInterfaceException,
    IllegalBindingException,
    IllegalLifeCycleException;
}
